@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014-2019 <tsujan2000@gmail.com>
  *
  * FeatherPad is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -24,11 +24,9 @@ namespace FeatherPad {
 // multi/single-line quote highlighting for bash.
 void Highlighter::SH_MultiLineQuote (const QString &text)
 {
-    static const QRegularExpression urlPattern ("[A-Za-z0-9_]+://[A-Za-z0-9_.+/\\?\\=~&%#\\-:]+|[A-Za-z0-9_.\\-]+@[A-Za-z0-9_\\-]+\\.[A-Za-z0-9.]+");
-
     int index = 0;
     QRegularExpressionMatch quoteMatch;
-    QRegularExpression quoteExpression ("\"|\'");
+    QRegularExpression quoteExpression = mixedQuoteMark;
     int initialState = currentBlockState();
     int prevState = previousBlockState();
 
@@ -50,8 +48,11 @@ void Highlighter::SH_MultiLineQuote (const QString &text)
     int hereDocDelimPos = -1;
     if (!curData->labelInfo().isEmpty()) // the label is delimStr
     {
-        QRegularExpression delim ("<<(?:\\s*)(\\\\{0,1}[A-Za-z0-9_]+)|<<(?:\\s*)(\'[A-Za-z0-9_]+\')|<<(?:\\s*)(\"[A-Za-z0-9_]+\")");
-        hereDocDelimPos = text.indexOf (delim);
+        hereDocDelimPos = text.indexOf (hereDocDelimiter);
+        while (hereDocDelimPos > -1 && isQuoted (text, hereDocDelimPos, true))
+        { // see isHereDocument()
+            hereDocDelimPos = text.indexOf (hereDocDelimiter, hereDocDelimPos + 2);
+        }
     }
 
     /* find the start quote */
@@ -70,7 +71,7 @@ void Highlighter::SH_MultiLineQuote (const QString &text)
         if (index >= 0)
         {
             /* ... distinguish between double and single quotes */
-            if (index == text.indexOf (quoteMark, index))
+            if (text.at (index) == quoteMark.pattern().at (0))
                 quoteExpression = quoteMark;
             else
                 quoteExpression.setPattern ("\'");
@@ -89,11 +90,11 @@ void Highlighter::SH_MultiLineQuote (const QString &text)
     while (index >= 0)
     {
         /* if the search is continued... */
-        if (quoteExpression.pattern() == "\"|\'")
+        if (quoteExpression == mixedQuoteMark)
         {
             /* ... distinguish between double and single quotes
                again because the quote mark may have changed */
-            if (index == text.indexOf (quoteMark, index))
+            if (text.at (index) == quoteMark.pattern().at (0))
                 quoteExpression = quoteMark;
             else
                 quoteExpression.setPattern ("\'");
@@ -150,7 +151,7 @@ void Highlighter::SH_MultiLineQuote (const QString &text)
         }
 
         /* the next quote may be different */
-        quoteExpression.setPattern ("\"|\'");
+        quoteExpression = mixedQuoteMark;
         index = text.indexOf (quoteExpression, index + quoteLength);
 
         /* skip escaped start quotes and all comments */
@@ -171,6 +172,7 @@ bool Highlighter::SH_SkipQuote (const QString &text, const int pos, bool isStart
     QTextCharFormat fi = format (pos);
     return (fi == neutralFormat // not needed
             || fi == commentFormat
+            || fi == urlFormat
             || fi == quoteFormat
             || fi == altQuoteFormat
             || fi == urlInsideQuoteFormat);
@@ -193,8 +195,8 @@ int Highlighter::formatInsideCommand (const QString &text,
             ++ indx;
         if (indx == text.length())
             break;
-        QString c = text.at (indx);
-        if (c == "\'")
+        const QChar c = text.at (indx);
+        if (c == '\'')
         {
             if (comment)
             {
@@ -231,7 +233,7 @@ int Highlighter::formatInsideCommand (const QString &text,
                 }
             }
         }
-        else if (c == "\"")
+        else if (c == '\"')
         {
             if (comment)
                 setFormat (indx, 1, commentFormat);
@@ -242,7 +244,7 @@ int Highlighter::formatInsideCommand (const QString &text,
             }
             ++ indx;
         }
-        else if (c == "$") // may start a new code block
+        else if (c == '$') // may start a new code block
         {
             if (comment)
             {
@@ -271,7 +273,7 @@ int Highlighter::formatInsideCommand (const QString &text,
                 }
             }
         }
-        else if (c == "(")
+        else if (c == '(')
         {
             if (doubleQuoted)
                 setFormat (indx, 1, quoteFormat);
@@ -286,7 +288,7 @@ int Highlighter::formatInsideCommand (const QString &text,
             }
             ++ indx;
         }
-        else if (c == ")") // may end a code block
+        else if (c == ')') // may end a code block
         {
             if (doubleQuoted)
                 setFormat (indx, 1, quoteFormat);
@@ -316,7 +318,7 @@ int Highlighter::formatInsideCommand (const QString &text,
             }
             ++ indx;
         }
-        else if (c == "#") // may be comment sign
+        else if (c == '#') // may be comment sign
         {
             if (comment)
                 setFormat (indx, 1, commentFormat);
@@ -437,7 +439,8 @@ bool Highlighter::SH_CmndSubstVar (const QString &text,
     {
         if (N == 0)
         { // search for the first code block (after the previous one is closed)
-            int start = text.indexOf (QRegularExpression ("\\$\\("), indx);
+            static const QRegularExpression codeBlockStart ("\\$\\(");
+            int start = text.indexOf (codeBlockStart, indx);
             if (start == -1 || format (start) == commentFormat)
                 goto FINISH;
             else
