@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014-2019 <tsujan2000@gmail.com>
  *
  * FeatherPad is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -46,12 +46,21 @@ void LineEdit::keyPressEvent (QKeyEvent *event)
         event->accept();
         return;
     }
-    /* since two line-ediits can be shown, Ctrl+K can't be used
-       as a QShortcut but can come here for clearing the text */
-    if (event->modifiers() == Qt::ControlModifier && event->key() == Qt::Key_K)
+    if (event->modifiers() == Qt::ControlModifier)
     {
-        clear();
-        returnPressed(); // for clearing found matches highlighting
+        if (event->key() == Qt::Key_Up || event->key() == Qt::Key_Down)
+        { // in case it belongs to a combo box
+            emit showComboPopup();
+            event->accept();
+            return;
+        }
+        /* since two line-ediits can be shown, Ctrl+K can't be used
+           as a QShortcut but can come here for clearing the text */
+        if (event->key() == Qt::Key_K)
+        {
+            clear();
+            emit returnPressed(); // for clearing found matches highlighting
+        }
     }
     QLineEdit::keyPressEvent (event);
 }
