@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2014-2020 <tsujan2000@gmail.com>
  *
  * FeatherPad is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -34,7 +34,7 @@ Loading::Loading (const QString& fname, const QString& charset, bool reload,
     posInLine_ (posInLine),
     forceUneditable_ (forceUneditable),
     multiple_ (multiple),
-    skipNonText_ (false)
+    skipNonText_ (true)
 {}
 /*************************/
 Loading::~Loading() {}
@@ -68,15 +68,23 @@ void Loading::run()
     QByteArray data;
     char c;
     qint64 charSize = sizeof (char); // 1
+    int num = 0;
     if (enforced)
     { // no need to check for the null character here
         while (file.read (&c, charSize) > 0)
-            data.append (c);
+        {
+            if (c == '\n' || c == '\r')
+                num = 0;
+            if (num < 500004) // a multiple of 4 (for UTF-16/32)
+                data.append (c);
+            else
+                forceUneditable_ = true;
+            ++num;
+        }
     }
     else
     {
         unsigned char C[4];
-        int num = 0;
         /* checking 4 bytes is enough to guess
            whether the encoding is UTF-16 or UTF-32 */
         while (num < 4 && file.read (&c, charSize) > 0)
@@ -111,7 +119,7 @@ void Loading::run()
             /* reading may still be possible */
             if (charset_.isEmpty() && !hasNull)
             {
-                int num = 0;
+                num = 5; // 4 characters are already read
                 while (file.read (&c, charSize) > 0)
                 {
                     if (c == '\0')
@@ -121,7 +129,7 @@ void Loading::run()
                             if (skipNonText_)
                             {
                                 file.close();
-                                emit completed();
+                                emit completed (QString(), QString(), "UTF-8"); // shows that a non-text file is skipped
                                 return;
                             }
                             hasNull = true;
@@ -144,18 +152,27 @@ void Loading::run()
                 if (skipNonText_ && hasNull && charset_.isEmpty())
                 {
                     file.close();
-                    emit completed();
+                    emit completed (QString(), QString(), "UTF-8");
                     return;
                 }
+                num = 0;
                 while (file.read (&c, charSize) > 0)
-                    data.append (c);
+                {
+                    if (c == '\n' || c == '\r')
+                        num = 0;
+                    if (num < 500004) // a multiple of 4 (for UTF-16/32)
+                        data.append (c);
+                    else
+                        forceUneditable_ = true;
+                    ++num;
+                }
             }
         }
     }
     file.close();
     if (skipNonText_ && hasNull && charset_.isEmpty())
     {
-        emit completed();
+        emit completed (QString(), QString(), "UTF-8");
         return;
     }
 
@@ -170,7 +187,7 @@ void Loading::run()
             charset_ = detectCharset (data);
     }
 
-    QTextCodec *codec = QTextCodec::codecForName (charset_.toUtf8()); // or charset.toStdString().c_str()
+    QTextCodec *codec = QTextCodec::codecForName (charset_.toUtf8()); // or charset_.toStdString().c_str()
     if (!codec) // prevent any chance of crash if there's a bug
     {
         charset_ = "UTF-8";
