@@ -36,14 +36,21 @@ void FPwin::toggleSyntaxHighlighting()
     int count = ui->tabWidget->count();
     if (count == 0) return;
 
+    bool enableSH = ui->actionSyntax->isChecked();
+    if (enableSH)
+        makeBusy(); // it may take a while with huge texts
+
     for (int i = 0; i < count; ++i)
     {
         TextEdit *textEdit = qobject_cast< TabPage *>(ui->tabWidget->widget (i))->textEdit();
-        syntaxHighlighting (textEdit, ui->actionSyntax->isChecked(), textEdit->getLang());
+        syntaxHighlighting (textEdit, enableSH, textEdit->getLang());
     }
 
-    if (QToolButton *langButton = ui->statusBar->findChild<QToolButton *>("langButton"))
-        langButton->setEnabled (ui->actionSyntax->isChecked());
+    if (TabPage *tabPage = qobject_cast< TabPage *>(ui->tabWidget->currentWidget()))
+        updateLangBtn (tabPage->textEdit());
+
+    if (enableSH)
+        QTimer::singleShot (0, this, &FPwin::unbusy);
 }
 /*************************/
 // Never returns an empty string; falls back to "url".
@@ -87,6 +94,8 @@ void FPwin::setProgLang (TextEdit *textEdit)
             progLan = "qmake";
         else if (fname.endsWith (".tr") || fname.endsWith (".t") || fname.endsWith (".roff"))
             progLan = "troff";
+        else if (fname.endsWith (".tex") || fname.endsWith (".ltx") || fname.endsWith (".latex") || fname.endsWith (".lyx"))
+            progLan = "LaTeX";
         else if (fname.endsWith (".xml", Qt::CaseInsensitive) || fname.endsWith (".svg", Qt::CaseInsensitive) || fname.endsWith (".qrc")
                  || fname.endsWith (".meta4", Qt::CaseInsensitive) || fname.endsWith (".metalink", Qt::CaseInsensitive)
                  /*|| fname.endsWith (".ui")*/ || fname.endsWith (".rdf") || fname.endsWith (".docbook") || fname.endsWith (".fnx")
@@ -102,10 +111,11 @@ void FPwin::setProgLang (TextEdit *textEdit)
         else if (fname.endsWith (".kvconfig")
                  || fname.endsWith (".service") || fname.endsWith (".mount") || fname.endsWith (".timer") // systemd related
                  || baseName == "sources.list" || baseName == "sources.list.save"
-                 || baseName == "mimeinfo.cache" || baseName == "mimeapps.list" || baseName.endsWith ("-mimeapps.list")
+                 || baseName == "mimeinfo.cache" || baseName == "defaults.list"
+                 || baseName == "mimeapps.list" || baseName.endsWith ("-mimeapps.list")
                  || fname.endsWith (".pls", Qt::CaseInsensitive))
              progLan = "config";
-        else if (fname.endsWith (".js"))
+        else if (fname.endsWith (".js") || fname.endsWith (".hx"))
             progLan = "javascript";
         else if (fname.endsWith (".qml"))
             progLan = "qml";
@@ -119,6 +129,10 @@ void FPwin::setProgLang (TextEdit *textEdit)
             progLan = "srt";
         else if (fname.endsWith (".theme"))
              progLan = "theme";
+        else if (fname.endsWith (".fountain"))
+             progLan = "fountain";
+        else if (fname.endsWith (".yml") || fname.endsWith (".yaml"))
+             progLan = "yaml";
         else if (fname.endsWith (".m3u", Qt::CaseInsensitive))
             progLan = "m3u";
         else if (fname.endsWith (".htm", Qt::CaseInsensitive) || fname.endsWith (".html", Qt::CaseInsensitive))
@@ -127,6 +141,8 @@ void FPwin::setProgLang (TextEdit *textEdit)
             progLan = "markdown";
         else if (fname.endsWith (".rst"))
             progLan = "reST";
+        else if (fname.endsWith (".dart"))
+            progLan = "dart";
         else if (baseName.startsWith ("makefile.", Qt::CaseInsensitive) && !baseName.endsWith (".txt", Qt::CaseInsensitive))
             progLan = "makefile";
         else if (baseName.compare ("CMakeLists.txt", Qt::CaseInsensitive) == 0)
@@ -169,7 +185,7 @@ void FPwin::setProgLang (TextEdit *textEdit)
                 progLan = "ruby";
             else if (mime == "text/x-lua")
                 progLan = "lua";
-            else if (mime == "text/x-python")
+            else if (mime.startsWith("text/x-python")) // it may be "text/x-python3"
                 progLan = "python";
             else if (mime == "application/x-perl")
                 progLan = "perl";
@@ -181,13 +197,15 @@ void FPwin::setProgLang (TextEdit *textEdit)
                 progLan = "qmake";
             else if (mime == "text/troff")
                 progLan = "troff";
+            else if (mime == "text/x-tex" || mime == "application/x-lyx")
+                progLan = "LaTeX";
             else if (mime == "application/xml" || mime == "image/svg+xml" || mime == "application/x-designer"
                      || mime == "application/metalink4+xml" || mime == "application/metalink+xml"
                      || mime == "application/x-gtk-builder" || mime == "text/rdf+xml" || mime == "application/rdf+xml"
                      || mime == "application/x-docbook+xml" || mime == "application/x-xbel" || mime == "text/feathernotes-fnx"
-                     || mime == "text/vnd.trolltech.linguist"
+                     || mime == "text/vnd.trolltech.linguist" || mime == "text/x-opml+xml"
                      || mime == "application/xspf+xml" || mime == "audio/x-ms-asx"
-                     || mime == "application/vnd.kde.kxmlguirc" || mime == "application/vnd.google-earth.kml+xml")
+                     || mime == "application/vnd.kde.kxmlguirc" || mime == "application/vnd.google-earth.kml+xml" || mime == "application/vnd.kde.kcfg")
                 progLan = "xml";
             else if (mime == "text/css")
                 progLan = "css";
@@ -197,7 +215,7 @@ void FPwin::setProgLang (TextEdit *textEdit)
                 progLan = "changelog";
             else if (mime == "application/x-desktop")
                 progLan = "desktop";
-            else if (mime == "audio/x-scpls")
+            else if (mime == "audio/x-scpls" || mime == "application/vnd.kde.kcfgc")
                 progLan = "config";
             else if (mime == "application/javascript")
                 progLan = "javascript";
@@ -209,13 +227,15 @@ void FPwin::setProgLang (TextEdit *textEdit)
                 progLan = "php";
             else if (mime == "application/x-theme")
                 progLan = "theme";
+            else if (mime == "application/x-yaml")
+                progLan = "yaml";
             else if (mime == "text/x-diff" || mime == "text/x-patch")
                 progLan = "diff";
             else if (mime == "text/html" || mime == "application/xhtml+xml")
                 progLan = "html";
             else if (mime == "text/markdown")
                 progLan = "markdown";
-            else if (mime == "audio/x-mpegurl")
+            else if (mime == "audio/x-mpegurl" || mime == "application/vnd.apple.mpegurl")
                 progLan = "m3u";
             else if (fname.endsWith (".conf") || fname.endsWith (".ini"))
                  progLan = "config"; // only if the mime type isn't found
@@ -271,19 +291,27 @@ void FPwin::syntaxHighlighting (TextEdit *textEdit, bool highlight, const QStrin
             Highlighter *highlighter = new Highlighter (textEdit->document(), progLan, start, end,
                                                         textEdit->hasDarkScheme(),
                                                         config.getShowWhiteSpace(),
-                                                        config.getShowEndings());
+                                                        config.getShowEndings(),
+                                                        config.getWhiteSpaceValue(),
+                                                        config.customSyntaxColors().isEmpty()
+                                                            ? textEdit->hasDarkScheme() ? config.darkSyntaxColors()
+                                                                                        : config.lightSyntaxColors()
+                                                            : config.customSyntaxColors());
             textEdit->setHighlighter (highlighter);
         }
         /* if the highlighter is created just now, it's necessary
            to wait until the text is completely loaded */
         QTimer::singleShot (0, textEdit, [this, textEdit]() {
             if (textEdit->isVisible())
+            {
+                formatTextRect(); // the text may be scrolled immediately after syntax highlighting (when reloading)
                 matchBrackets(); // in case the cursor is beside a bracket when the text is loaded
+            }
             connect (textEdit, &TextEdit::updateBracketMatching, this, &FPwin::matchBrackets);
             /* visible text may change on block removal */
             connect (textEdit, &QPlainTextEdit::blockCountChanged, this, &FPwin::formatOnBlockChange);
-            connect (textEdit, &TextEdit::updateRect, this, &FPwin::formatVisibleText);
-            connect (textEdit, &TextEdit::resized, this, &FPwin::formatOnResizing);
+            connect (textEdit, &TextEdit::updateRect, this, &FPwin::formatTextRect);
+            connect (textEdit, &TextEdit::resized, this, &FPwin::formatTextRect);
             /* this is needed when the whole visible text is pasted */
             connect (textEdit->document(), &QTextDocument::contentsChange, this, &FPwin::formatOnTextChange);
         });
@@ -291,8 +319,8 @@ void FPwin::syntaxHighlighting (TextEdit *textEdit, bool highlight, const QStrin
     else if (Highlighter *highlighter = qobject_cast< Highlighter *>(textEdit->getHighlighter()))
     {
         disconnect (textEdit->document(), &QTextDocument::contentsChange, this, &FPwin::formatOnTextChange);
-        disconnect (textEdit, &TextEdit::resized, this, &FPwin::formatOnResizing);
-        disconnect (textEdit, &TextEdit::updateRect, this, &FPwin::formatVisibleText);
+        disconnect (textEdit, &TextEdit::resized, this, &FPwin::formatTextRect);
+        disconnect (textEdit, &TextEdit::updateRect, this, &FPwin::formatTextRect);
         disconnect (textEdit, &QPlainTextEdit::blockCountChanged, this, &FPwin::formatOnBlockChange);
         disconnect (textEdit, &TextEdit::updateBracketMatching, this, &FPwin::matchBrackets);
 
@@ -321,32 +349,19 @@ void FPwin::formatOnTextChange (int /*position*/, int charsRemoved, int charsAdd
     {
         /* wait until the document's layout manager is notified about the change;
            otherwise, the end cursor might be out of range in formatTextRect() */
-        QTimer::singleShot (0, this, &FPwin::formatOnResizing);
+        QTimer::singleShot (0, this, &FPwin::formatTextRect);
     }
 }
 /*************************/
 void FPwin::formatOnBlockChange (int/* newBlockCount*/) const
 {
-    formatOnResizing();
+    formatTextRect();
 }
 /*************************/
-void FPwin::formatVisibleText (const QRect &rect, int dy) const
-{
-    if (dy == 0) return;
-    formatTextRect (rect);
-}
-/*************************/
-void FPwin::formatOnResizing() const
-{
-    int index = ui->tabWidget->currentIndex();
-    if (index == -1) return;
-
-    TextEdit *textEdit = qobject_cast< TabPage *>(ui->tabWidget->widget (index))->textEdit();
-    formatTextRect (textEdit->rect());
-}
-/*************************/
-void FPwin::formatTextRect (QRect rect) const
+void FPwin::formatTextRect() const
 {
+    /* It's supposed that this function is called for the current tab.
+       That isn't always the case and won't do any harm if it isn't. */
     if (TabPage *tabPage = qobject_cast<TabPage*>(ui->tabWidget->currentWidget()))
     {
         TextEdit *textEdit = tabPage->textEdit();
@@ -355,7 +370,7 @@ void FPwin::formatTextRect (QRect rect) const
 
         QPoint Point (0, 0);
         QTextCursor start = textEdit->cursorForPosition (Point);
-        Point = QPoint (rect.width(), rect.height());
+        Point = QPoint (textEdit->width(), textEdit->height());
         QTextCursor end = textEdit->cursorForPosition (Point);
 
         highlighter->setLimit (start, end);
